//////////////////////////////////////////////////////////////////////////////
// ***** BEGIN LICENSE BLOCK *****
// Version: MPL 1.1/GPL 2.0/LGPL 2.1
//
// The contents of this file are subject to the Mozilla Public License Version
// 1.1 (the "License"); you may not use this file except in compliance with
// the License. You may obtain a copy of the License at
// http://www.mozilla.org/MPL/
//
// Software distributed under the License is distributed on an "AS IS" basis,
// WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
// for the specific language governing rights and limitations under the
// License.
//
// The Original Code is CamerAH.pas .
//
// The Initial Developer of the Original Code is Jens Biermann.
// Portions created by the Initial Developer are Copyright (C) 2011
// the Initial Developer. All Rights Reserved.
//
// Contributor(s):
//
// Alternatively, the contents of this file may be used under the terms of
// either the GNU General Public License Version 2 or later (the "GPL"), or
// the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
// in which case the provisions of the GPL or the LGPL are applicable instead
// of those above. If you wish to allow use of your version of this file only
// under the terms of either the GPL or the LGPL, and not to allow others to
// use your version of this file under the terms of the MPL, indicate your
// decision by deleting the provisions above and replace them with the notice
// and other provisions required by the GPL or the LGPL. If you do not delete
// the provisions above, a recipient may use your version of this file under
// the terms of any one of the MPL, the GPL or the LGPL.
//
// ***** END LICENSE BLOCK *****
//
//  Release 01-05-2011
//  - afterRotationSensitivity, es wird die Geschwidigkeit gemessen und ab dem Grenzwert
//    "afterRotationSensitivity" luft das afterRotation oder nicht
//  - prft in projection ob sich der RenderKontext gendert hat 
//    und setzt ggf IsToRender auf True
//  - neuer Cursor fuer DepthMove
//
//  Release 02-04-2011
//  - Selection mit PickingMatrix, rmSelection
//  - ColorPicking, rmColorPick
//  - BugFix Zoom
//  - FillLevel
//  - Indentity fr Camera- und Projectionsmatrix CM_Identity, PM_Identity
//
//  Release 29-03-2011
//  - afterRotation
//  - CursorIcons
//
//  Release 23-03-2011
//  - Rotation um X und Y-Achse
//  - Rotation um die einzelnen Achsen
//  - Rotationsspeed
//  - Move in X und Y-Richtung und in die Tiefe
//  - Movespeed
//  - Verschieben des Rotationpunktes (oder nicht)
//  - Verwalten der Perspectiv-Modi
//  - Ausgleich der Verzerrung (Hhe, Breite des Fensters) z.B. beim Resize
//  - mehrfacher Fensterzoom (+/-)
//  - Setzen der Ansichten auf XY, XZ, ...
//  - ArcBall-Rotation
//////////////////////////////////////////////////////////////////////////////

unit CamerAH;

interface

uses
  dglOpenGL, CamerAHGeometry, Controls, CamerAHBasis, Windows, CamerAHUtils
  //, CamerAHSkybox
  ;

type

  TglCamerAH = class(TglCamerAHBasis)
  private
    //SkyBox: TglSkybox;

    ClipPlanes: array of TClipPlane;
    IsToRender: Boolean;
    FSpeedRot: Single;
    FSpeedMove: Single;
    lastPoint, nexttolastPoint: TPoint;
    IsafterRotationStopped: Boolean;
    afterRotationLerp: Single;
    FFillLevel: Single;
	  tmpDC: HDC;
	  tmpRC: HGLRC;
    StopWatch: TStopWatch;
    procedure afterRotationStart;
    procedure RotArcBall(X, Y: Single);
    procedure Move(MovetAxis: CamerAHGeometry.TVector; XY: Integer);
    procedure RenderMouseRect;
    procedure Rotate(RotAxis: CamerAHGeometry.TVector; XY: Integer);
    procedure SelectionColor(X, Y: Integer);
    procedure SetFillLevel(const Value: Single);
    procedure SetSpeedRot(const Value: Single);
    procedure SetSpeedMove(const Value: Single);
    procedure SetRotMode(const Value: TRotMode);
    procedure Sphere;
    procedure Zoom_plus(Rec: TRect);
    procedure Zoom_minus;
    procedure RotateAngle(RotAxis: CamerAHGeometry.TVector; Angle: Single);
    function  RadiusArcBall: Single;
    procedure afterRotationSetPoints(X, Y: Integer);
    procedure afterRotationStop(X, Y: Integer);
  protected
  public
    ArcBallRadius: Single;
    ArcBallStart, ArcBallEnd: CamerAHGeometry.TVector;
    RotAxis1, RotAxis2: CamerAHGeometry.TVector;
    Mouse: TglMouse;
    MemoPoints: array of CamerAHGeometry.TVector;
    IsafterRotation: Boolean;
    afterRotationdamping,
    afterRotationSensitivity: Single;
    SelectNumber: Integer;
    constructor Create;
    destructor Destroy; override;
    procedure Adjust(V: TView = XY; Invers: Boolean = False);
    procedure Apply;
    procedure ApplyInvers;
    function  afterRotation: Boolean;
    procedure SetProjection(left, right, bottom, top, zNear, zFar: Single);
    procedure PositionCamera(Camera, Center, Up, PoR: CamerAHGeometry.TVector); overload;
    procedure PositionCamera(Ca0,Ca1,Ca2,  Ce0,Ce1,Ce2,  Up0,Up1,Up2,  PoR0,PoR1,PoR2: Single); overload;
    procedure ProjOrtho(left, right, bottom, top, zNear, zFar: Single);
    procedure ProjFrustum(left, right, bottom, top, zNear, zFar: Single);
    procedure ProjPerspectiv(FOV, Aspect, ZNear, ZFar: Single);
    procedure Projection(PMode: TProjection);
    procedure SelectionStart(X, Y: Single);
    function  SelectionResult(SearchDepth: Single = 0.1): TSelectionResults;
    procedure Resize;
    procedure RotateStart(X, Y: Integer);
    function  RotateMove(X, Y: Integer): boolean;
    procedure RotateStop;
    function  ClipPlane: TClipPlane;

    property FillLevel: Single read FFillLevel write SetFillLevel;
    property RotMode: TRotMode read FRotMode write SetRotMode;
    property SpeedRot: Single read FSpeedRot write SetSpeedRot;
    property SpeedMove: Single read FSpeedMove write SetSpeedMove;
  end;


implementation

uses
  Classes, Types, Dialogs, CamerAHTypes, Math;

function TglCamerAH.ClipPlane: TClipPlane;
begin
  Result := ClipPlanes[Length(ClipPlanes)-1];
end;

constructor TglCamerAH.Create;
begin
  inherited;
  Mouse := TglMouse.Create;
  StopWatch := TStopWatch.Create;
  //SkyBox := TglSkybox.Create;
  IsafterRotation := False;
  SpeedRot := 1;
  SpeedMove:= 1;
  RotMode  := rmFlexAxis;
  ProjectionMode := pjOrtho;
  IsToRender := True;
  SetLength(ClipPlanes, 1);
  afterRotationdamping := 0.85;
  afterRotationSensitivity := 0.5;
  FillLevel := 1;
  tmpDC := 0;
  tmpRC := 0;  
end;

destructor TglCamerAH.Destroy;
begin
  //SkyBox.Free;
  StopWatch.Free;
  Mouse.Free;
  inherited;
end;

procedure TglCamerAH.Adjust(V: TView = XY; Invers: Boolean = False);
var
  eye, center, up, VM: CamerAHGeometry.TVector;
  L: Single;
begin
  L:= DistanceCameraCenter;
  if Invers then
    L := -L;

  case V of
    XY:
      begin
        eye := VectorMake(0, 0, L);
        up  := VectorMake(0, 1, 0);
      end;
    XZ:
      begin
        eye := VectorMake(0, -L, 0);
        up  := VectorMake(0, 0, 1);
      end;
    YX:
      begin
        eye := VectorMake(0, 0, L);
        up  := VectorMake(1, 0, 0);
      end;
    YZ:
      begin
        eye := VectorMake(L, 0, 0);
        up  := VectorMake(0, 0, 1);
      end;
    ZX:
      begin
        eye := VectorMake(0, L, 0);
        up  := VectorMake(1, 0, 0);
      end;
    ZY:
      begin
        eye := VectorMake(L, 0, 0);
        up  := VectorMake(0, 1, 0);
      end;
  end;

  eye := VectorAdd(eye, PointofView);
  center := PointofView;
  PositionCamera(eye, center, up, PointofRotation);
end;

procedure TglCamerAH.Projection(PMode: TProjection);
var
  FL: Single;
begin
  if ProjectionMode <> PMode then
    IsToRender := True;
  if (wglGetCurrentDC <> tmpDC) or (wglGetCurrentContext <> tmpRC) then
  begin
	IsToRender := True;
	tmpDC := wglGetCurrentDC;
	tmpRC := wglGetCurrentContext;
  end;

  ProjectionMode := PMode;

  FL := FillLevel;
  with ClipPlanes[Length(ClipPlanes)-1] do
  case ProjectionMode of
    pjOrtho:      ProjOrtho(left/FL, right/FL, bottom/FL, top/FL, zNear, zFar);
    pjPerspectiv: ProjFrustum(left/FL, right/FL, bottom/FL, top/FL, zNear, zFar);
  end;

  if (RotMode = rmArcBall) and EnableRotion then
    Sphere;

  //SkyBox.DrawScene;
end;

{$REGION 'Selection'}
procedure TglCamerAH.SelectionStart(X, Y: Single);
// http://wiki.delphigl.com/index.php/Tutorial_Objektselektion
var
  viewport: CamerAHTypes.TVector4i;
begin
  glGetIntegerv(GL_VIEWPORT, @viewport);
  glSelectBuffer(512, @SelectBuffer);
  glRenderMode(GL_SELECT);
  glInitNames;
  glPushName(0);

  tmpProjMatrix := ProjMatrix;
  ProjMatrix :=
    MatrixMultiply(ProjMatrix, CreatePickMatrix(X, viewport[3] - Y, 4, 4, viewport));
  glMatrixMode(GL_PROJECTION);
  glLoadMatrixf(@ProjMatrix);
  IsToRender := False;
end;

function TglCamerAH.SelectionResult(SearchDepth: Single = 0.1): TSelectionResults;
type
  PHit = ^RHit;
  RHit = record
    ZValue: Single;
    No: Integer;
  end;
var
  Hits, Count, i: Integer;
  D: Single;
  n: Extended;
  HitList: TList;
  H: PHit;

  function CompareHit(Item1 : Pointer; Item2 : Pointer): Integer;
  begin
    Result := CompareValue(PHit(Item1).ZValue, PHit(Item2).ZValue);
  end;
begin
  HitList := TList.Create;
  Count := 0;
  D := Abs(ClipPlane.zfar - ClipPlane.znear);
  n := IntPower(2, 32) - 1;
  if SearchDepth < 0 then
    SearchDepth := 0;

  Hits := glRenderMode(GL_RENDER);
  SetLength(Result, Hits);

  for i := 0 to Hits - 1 do
  begin
    New(H);
    H^.ZValue := D * SelectBuffer[(i*4)+1] / n;
    H^.No := SelectBuffer[(i*4)+3];
    HitList.Add(H);
  end;

  HitList.Sort(@CompareHit);
  //HitList.Sort(CompareHit); // for older Delphi-Versions

  for i := 0 to HitList.Count - 1 do
    if  (PHit(HitList[i]).ZValue <= PHit(HitList[0]).ZValue + SearchDepth) then
    begin
      Result[i] := PHit(HitList[i]).No;
      Inc(Count);
    end;

  SetLength(Result, Count);

  ProjMatrix := tmpProjMatrix;
  IsToRender := True;

  for i := 0 to HitList.Count - 1 do
  begin
    H := HitList.Items[i];
    Dispose(H);
  end;

  HitList.Free;
end;
{$ENDREGION}

procedure TglCamerAH.SetProjection(left, right, bottom, top, zNear, zFar: Single);
var
  i: Integer;
begin
  i := Length(ClipPlanes) -1;
  ClipPlanes[i].left   := left;
  ClipPlanes[i].right  := right;
  ClipPlanes[i].bottom := bottom;
  ClipPlanes[i].top    := top;
  ClipPlanes[i].znear  := zNear;
  ClipPlanes[i].zfar   := zFar;
  IsToRender := True;
end;

procedure TglCamerAH.PositionCamera(Camera, Center, Up, PoR: CamerAHGeometry.TVector);
begin
  CameraMatrix := CreateLookAtMatrix(Camera, Center, Up);
  PointofCamera := Camera;
  PointofView := Center;
  PointofRotation := PoR;
  PointofOrgin := NullHmgVector;
end;

procedure TglCamerAH.PositionCamera(Ca0,Ca1,Ca2,  Ce0,Ce1,Ce2,  Up0,Up1,Up2,  PoR0,PoR1,PoR2: Single);
begin
  PositionCamera(
    VectorMake(Ca0,  Ca1,  Ca2),
    VectorMake(Ce0,  Ce1,  Ce2),
    VectorMake(Up0,  Up1,  Up2),
    VectorMake(PoR0, PoR1, PoR2));
end;

procedure TglCamerAH.ProjOrtho(left, right, bottom, top, zNear, zFar: Single);
begin
  if IsToRender then
  begin
    ProjMatrix := CreateOrthoMatrix(
                    left   * ScreenAdjustWidth, right * ScreenAdjustWidth,
                    bottom * ScreenAdjustHeight, top  * ScreenAdjustHeight,
                    zNear, zFar);
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(@ProjMatrix);
    IsToRender := False;
  end;
  RenderMouseRect;
end;

procedure TglCamerAH.ProjFrustum(left, right, bottom, top, zNear, zFar: Single);
begin
  if IsToRender then
  begin
    ProjMatrix := CreateMatrixFromFrustum(
                    left   * ScreenAdjustWidth, right * ScreenAdjustWidth,
                    bottom * ScreenAdjustHeight, top  * ScreenAdjustHeight,
                    zNear, zFar);
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(@ProjMatrix);
    IsToRender := False;
  end;
  RenderMouseRect;
end;

procedure TglCamerAH.ProjPerspectiv(FOV, Aspect, ZNear, ZFar: Single);
begin
  if IsToRender then
  begin
    ProjMatrix := CreatePerspectiveMatrix(FOV, Aspect, ZNear, ZFar);
    glMatrixMode(GL_PROJECTION);
    glLoadMatrixf(@ProjMatrix);
    IsToRender := False;
  end;
  RenderMouseRect;
end;

{$REGION 'afterRotation'}
procedure TglCamerAH.afterRotationSetPoints(X, Y: Integer);
begin
  nexttolastPoint := lastPoint;
  StopWatch.Start;
  lastPoint := Point(X, Y);
end;

function TglCamerAH.afterRotation: Boolean;
var
  XY: CamerAHGeometry.TVector;
begin
  if IsafterRotation and not IsafterRotationStopped then
  begin
    XY := VectorLerp( VectorMake(nexttolastPoint.X, nexttolastPoint.Y, 0),
                      VectorMake(lastPoint.X, lastPoint.Y, 0), afterRotationLerp);
    afterRotationLerp := afterRotationLerp * afterRotationdamping;

    if (afterRotationLerp < 0.001 ) then
    begin
      IsafterRotationStopped := True;
      Result := False;
    end
    else
    begin
      Result := True;
      RotArcBall(XY[0], XY[1]);
    end;
  end
  else
  begin
    Result := False;
  end;
end;

procedure TglCamerAH.afterRotationStart;
var
  l: Integer;
  Time: Int64;
  velocity: Double;
begin
  StopWatch.Stop;
  Time := StopWatch.StoppedTime;

  xStart := nexttolastPoint.X;
  yStart := nexttolastPoint.Y;
  l := Round( Sqrt( Sqr(lastPoint.X - xStart) + Sqr(lastPoint.Y - yStart) ) );
  velocity := l / Time;

  if IsafterRotation and (velocity > afterRotationSensitivity) then
  begin
    IsafterRotationStopped := False;

    afterRotationLerp := 0.03 * l;
    if afterRotationLerp > 30 then
      afterRotationLerp := 30;
  end;
end;

procedure TglCamerAH.afterRotationStop(X, Y: Integer);
begin
  IsafterRotationStopped := True;
  nexttolastPoint := Point(X, Y);
  lastPoint := Point(X, Y);
end;

{$ENDREGION}

procedure TglCamerAH.Apply;
begin
  glMatrixMode(GL_MODELVIEW);
  glLoadMatrixf(@CameraMatrix);
end;

procedure TglCamerAH.ApplyInvers;
var
  M: CamerAHGeometry.TMatrix;
begin
  M := MatrixInvert(CameraMatrix);
  M[3] := WHmgVector;
  glMultMatrixf(@M);
end;

{$REGION 'ArcBall'}
function TglCamerAH.RadiusArcBall: Single;
var
  S: Single;
begin
  if ArcBallRadius <= 0 then
    Result := VectorLength(PointofRotation);
  if ScreenWidth * AdjustWidth < ScreenHeight * AdjustHeight then
    S := ScreenWidth * AdjustWidth
  else
    S := ScreenHeight * AdjustHeight;
  if Result < S/4 then
    Result := S/4;
  if Result > S/2 then
    Result := S/2;
end;

procedure TglCamerAH.RotArcBall(X, Y: Single);
var
  PoR,
  Axis: CamerAHGeometry.TVector;
  W, X1, Y1: Single;

  procedure mapToSphere(Xm, Ym: Double; var NewVec: CamerAHGeometry.TVector);
  var
    VX, VY, VZ: CamerAHGeometry.TVector;
    a, Z: Double;
  begin
    a := Sqr(Xm) + Sqr(Ym);
    if Sqrt(a) > RadiusArcBall then
    begin
      Ym := Ym * RadiusArcBall / Sqrt(a);
      Xm := Xm * RadiusArcBall / Sqrt(a);
      Z := 0;
    end
    else
    begin
      Z  := Sqrt(Sqr(RadiusArcBall) - a);
    end;

    VX := VectorScale(VectorNormalize(XVectors), Xm);
    VY := VectorScale(VectorNormalize(YVectors), Ym);
    VZ := VectorScale(VectorNormalize(ZVectors), Z);
    NewVec := VectorAdd2([VX, VY, VZ]);
  end;

begin
  if X > ScreenWidth then
    X := ScreenWidth
  else if X < 0 then
    X := 0;
  if Y > ScreenHeight then
    Y := ScreenHeight
  else if Y < 0 then
    Y := 0;

  PoR := OGLPanelPos(PointofRotation);
  X1 := (xStart - PoR[0]) * AdjustWidth;
  Y1 := ((ScreenHeight - yStart) - PoR[1]) * AdjustHeight;
  mapToSphere(X1, Y1, ArcBallStart);

  X1 := (X - PoR[0]) * AdjustWidth;
  Y1 := ((ScreenHeight - Y) - PoR[1]) * AdjustHeight;
  mapToSphere(X1, Y1, ArcBallEnd);

  Axis := VectorCrossProduct(ArcBallEnd, ArcBallStart);

  if VectorLength(Axis) < EPSILON then
    Exit;
  RotAxis1 := Axis;
  W := CamerAHGeometry.ArcCos(VectorAngleCosine(AffineVectorMake(ArcBallStart), AffineVectorMake(ArcBallEnd)));

  AddVector(ArcBallStart, PointofRotation);
  AddVector(ArcBallEnd, PointofRotation);
  RotateAngle(RotAxis1, W);
end;
{$ENDREGION}

procedure TglCamerAH.RenderMouseRect;
var
  V1, V2: CamerAHGeometry.TVector;
begin
  if (RotMode = rmZoomPlus) and Mouse.IsLupeZiehen and EnableRotion then
  with Mouse do
  begin
    glPushAttrib(GL_ALL_ATTRIB_BITS);
    glPushmatrix;
    // Text immer zum benutzer ausrichten, also nur die drehung und nicht die
    // verschiebung zurcknehmen
    ApplyInvers;

    V1 := OGLPos(FAktivFlche.left, FAktivFlche.Top, 0);
	V1[2] := ClipPlane.znear;
    V2 := OGLPos(FAktivFlche.right, FAktivFlche.bottom, 0);
	V2[2] := ClipPlane.znear;


//    glShadeModel(GL_SMOOTH);
//    glDisable(GL_DEPTH_TEST);
//    glEnable(GL_LINE_STIPPLE);
//    glLineStipple(4, $FFC0);
//
//    glBegin(GL_LINE_LOOP);
//      glColor3f(1, 1, 1);
//      glVertex3f(V1[0], V1[1], V1[2]);
//      glVertex3f(V2[0], V1[1], V1[2]);
//      glVertex3f(V2[0], V2[1], V1[2]);
//      glVertex3f(V1[0], V2[1], V1[2]);
//    glEnd;

    glDisable(GL_DEPTH_TEST);
    glEnable(GL_BLEND);
    glBlendFunc(GL_ONE_MINUS_SRC_ALPHA, GL_SRC_ALPHA);
    glBegin(GL_QUADS);
      glColor4f(0.3, 0, 0, 0.1);
      glVertex3f(V1[0], V1[1], V1[2]);
      glVertex3f(V1[0], V2[1], V1[2]);
      glVertex3f(V2[0], V2[1], V1[2]);
      glVertex3f(V2[0], V1[1], V1[2]);
    glEnd;
    glDisable(GL_BLEND);

    glPopmatrix;
    glPopAttrib;
  end;
end;

procedure TglCamerAH.Resize;
begin
  IsToRender := True;
end;

procedure TglCamerAH.Move(MovetAxis: CamerAHGeometry.TVector; XY: Integer);
var
  V: CamerAHGeometry.TVector;
  M: CamerAHGeometry.TMatrix;
  i: Integer;
begin
  V := MoveVector(MovetAxis, Round(XY * SpeedMove));
  M := CreateTranslationMatrix( V );
  CM_Multiply(M);

  PointofCamera := VectorSubtract(PointofCamera, V);
  PointofView := VectorSubtract(PointofView, V);
  for i := 0 to Length(MemoPoints) - 1 do
    MemoPoints[i] := VectorSubtract(MemoPoints[i], V);
  if not IsMovePoR then
    PointofRotation := VectorSubtract(PointofRotation, V);
end;

procedure TglCamerAH.RotateAngle(RotAxis: CamerAHGeometry.TVector; Angle: Single);
var
  VC, VV: CamerAHGeometry.TVector;
  VMemo: array of CamerAHGeometry.TVector;
  M: CamerAHGeometry.TMatrix;
  i: Integer;
begin
  VV := VectorSubtract(PointofView, PointofRotation);
  VC := VectorSubtract(PointofCamera, PointofRotation);
  SetLength(VMemo, Length(MemoPoints));
  for i := 0 to Length(VMemo) - 1 do
    VMemo[i] := VectorSubtract(MemoPoints[i], PointofRotation);

  M := CreateTranslationMatrix((PointofRotation));
  CM_Multiply(M);

  M := CreateRotationMatrix(RotAxis, Angle);
  CM_Multiply(M);
  VC := VectorTransform(VC, MatrixInvert(M));
  VV := VectorTransform(VV, MatrixInvert(M));
  for i := 0 to Length(VMemo) - 1 do
    VMemo[i] := VectorTransform(VMemo[i], MatrixInvert(M));

  M := CreateTranslationMatrix(VectorNegate(PointofRotation));
  CM_Multiply(M);

  PointofCamera := VectorAdd(PointofRotation, VC);
  PointofView := VectorAdd(PointofRotation, VV);
  for i := 0 to Length(VMemo) - 1 do
    MemoPoints[i] := VectorAdd(PointofRotation, VMemo[i]);
end;

procedure TglCamerAH.Rotate(RotAxis: CamerAHGeometry.TVector; XY: Integer);
begin
  RotateAngle(RotAxis, RotAngle(XY) * SpeedRot);
end;

procedure TglCamerAH.Zoom_minus;
begin
  if Length(ClipPlanes) > 1 then
  begin
    SetLength(ClipPlanes, Length(ClipPlanes) -1);
    IsToRender := True;
  end;
end;

procedure TglCamerAH.Zoom_plus(Rec: TRect);
var
  len, high,
  Rtop, Rbottom, Rleft, Rright,
  l,r, t,b, n,f,
  SRH, SRW: Single;
begin
  with ClipPlanes[Length(ClipPlanes)-1] do
  begin
    len := right -  left;
    high := top - bottom;

    SRH := 1;
    SRW := 1;
    if ScreenHeight > ScreenWidth then
      SRH := ScreenRelation
    else
      SRW := 1/ScreenRelation;

    if RectWidth(Rec)/RectHeight(Rec) > ScreenRelation then
    begin
      Rleft   := Rec.Left*SRW;
      Rright  := Rec.Right*SRW;
      Rtop    := Rec.Top*SRH + RectHeight(Rec)*SRH/2 - RectWidth(Rec)*SRW /2;
      Rbottom := Rec.Top*SRH + RectHeight(Rec)*SRH/2 + RectWidth(Rec)*SRW /2;
    end
    else
    begin
      Rleft   := Rec.Left*SRW + RectWidth(Rec)*SRW/2 - RectHeight(Rec)*SRH/2;
      Rright  := Rec.Left*SRW + RectWidth(Rec)*SRW/2 + RectHeight(Rec)*SRH/2;
      Rtop    := Rec.Top*SRH;
      Rbottom := Rec.Bottom*SRH;
    end;

    l := (left   +         Rleft   * AdjustWidth*FillLevel);
    r := (right  - (len -  Rright  * AdjustWidth*FillLevel));
    t := (top    -         Rtop    * AdjustHeight*FillLevel);
    b := (bottom + (high - Rbottom * AdjustHeight*FillLevel));
    n := znear;
    f := zfar;
  end;

  SetLength(ClipPlanes, Length(ClipPlanes) +1 );
  SetProjection(l, r, b, t, n, f);
end;

procedure TglCamerAH.RotateStart(X, Y: Integer);
begin
  case RotMode of
    rmNothing: Exit;
    rmFlexAxis: ;
    rmArcBall: ;
    rmMove: ;
    rmDepthMove: ;
    rmRotX: ;
    rmRotY: ;
    rmRotZ: ;
    rmRotCoordX: ;
    rmRotCoordY: ;
    rmRotCoordZ: ;
    rmZoomPlus: Mouse.An(X, Y, [ssLeft], 0,0);
    rmSelect: SelectionStart(X, Y);
    rmColorPick: Colorpicking(X, Y);
  end;
  xStart := X;
  yStart := Y;
  EnableRotion := True;
  afterRotationStop(X, Y);
  afterRotationSetPoints(X, Y);
end;

function TglCamerAH.RotateMove(X, Y: Integer): boolean;
var
  xMoved,
  yMoved: Integer;
begin
  Result := False;
  if ((X = 0) and (Y = 0)) or not EnableRotion then
    Exit;

  xMoved := xStart - X;
  yMoved := yStart - Y;

  case RotMode of
    rmRotX:
      begin
        Rotate(RotAxis1, xMoved);
        Rotate(RotAxis1, yMoved);
        Result := True;
      end;
    rmRotY:
      begin
        Rotate(RotAxis1, xMoved);
        Rotate(RotAxis1, yMoved);
        Result := True;
      end;
    rmRotZ:
      begin
        Rotate(RotAxis1, xMoved);
        Rotate(RotAxis1, yMoved);
        Result := True;
      end;
    rmRotCoordX:
      begin
        Rotate(RotAxis1, xMoved);
        Rotate(RotAxis1, yMoved);
        Result := True;
      end;
    rmRotCoordY:
      begin
        Rotate(RotAxis1, xMoved);
        Rotate(RotAxis1, yMoved);
        Result := True;
      end;
    rmRotCoordZ:
      begin
        Rotate(RotAxis1, xMoved);
        Rotate(RotAxis1, yMoved);
        Result := True;
      end;
    rmFlexAxis:
      begin
        RotAxis1 := YVectors;
        Rotate(RotAxis1, xMoved);

        RotAxis2 := XVectors;
        Rotate(RotAxis2, yMoved);
        Result := True;
      end;
    rmArcBall:
      begin
        afterRotationSetPoints(X, Y);
        RotArcBall(X, Y);
        Result := True;
      end;
    rmMove:
      begin
        Move(XVectors, -xMoved);
        Move(YVectors,  yMoved);
        Result := True;
      end;
    rmDepthMove:
      begin
        Move(ZVectors, yMoved);
        Result := True;
      end;
    rmZoomPlus:
      begin
        Mouse.Bewegen(X, Y);
        Result := True;
      end;
  end;

  xStart := xStart - xMoved;
  yStart := yStart - yMoved;
end;

procedure TglCamerAH.RotateStop;
begin
  EnableRotion := False;

  case RotMode of
    rmNothing:;
    rmFlexAxis: ;
    rmArcBall: afterRotationStart;
    rmMove: ;
    rmDepthMove: ;
    rmRotX: ;
    rmRotY: ;
    rmRotZ: ;
    rmRotCoordX: ;
    rmRotCoordY: ;
    rmRotCoordZ: ;
    rmZoomPlus: Zoom_plus(Mouse.AktivFlche);
  end;
end;

procedure TglCamerAH.SetRotMode(const Value: TRotMode);
begin
  inherited;
  case FRotMode of
    rmFlexAxis:;
    rmArcBall: ;
    rmMove: ;
    rmDepthMove: ;
    rmRotX: RotAxis1 := XVectors;
    rmRotY: RotAxis1 := YVectors;
    rmRotZ: RotAxis1 := ZVectors;
    rmRotCoordX: RotAxis1 := XHmgVector;
    rmRotCoordY: RotAxis1 := YHmgVector;
    rmRotCoordZ: RotAxis1 := ZHmgVector;
    rmZoomMinus: Zoom_minus;
  end;
end;

procedure TglCamerAH.SetSpeedMove(const Value: Single);
begin
  FSpeedMove := Value;
end;

procedure TglCamerAH.SetSpeedRot(const Value: Single);
begin
  FSpeedRot := Value;
end;

procedure TglCamerAH.SelectionColor(X, Y: Integer);
var
  R,G,B,A: GLubyte;
begin
  Colorpicking(X, Y, R,G,B,A);
  SelectNumber := R;
end;

procedure TglCamerAH.SetFillLevel(const Value: Single);
begin
  FFillLevel := Value;
  IsToRender := True;
end;

procedure TglCamerAH.Sphere;
var
  q : PGLUquadric;
  V: array[0..100] of CamerAHGeometry.TVector;
  i: Integer;

  procedure XYArc(Radius : Single; startAngle, stopAngle : Single; nbSegments : Integer;
    const center, point : CamerAHGeometry.TVector);
  // aus GLScene: unit GLNodes;
  var
     i : Integer;
     f : Single;
     s, c,
     WX, WY, WZ: Single;
     P: CamerAHGeometry.TVector;
  begin
    P := VectorSubtract(point, center);
    startAngle := CamerAHGeometry.DegToRad(startAngle);
    stopAngle  := CamerAHGeometry.DegToRad(stopAngle);
    WZ := CamerAHGeometry.ArcCos( VectorAngleCosine(XVector, AffineVectorMake(P[0], P[1], 0)) );

    f := (stopAngle - startAngle) / nbSegments;
    for i := 0 to nbSegments do
    begin
       CamerAHGeometry.SinCos(i * f + startAngle, s, c);
       V[i][0] := center[0] + Radius * c;
       V[i][1] := center[1] + Radius * s;
       V[i][2] := center[2];
       //V[i] := VectorMake( VectorRotateAroundZ(AffineVectorMake(V[i]), -WZ));
    end;
  end;

begin
  glPushAttrib(GL_ALL_ATTRIB_BITS);
  glPushMatrix;
    glPolygonmode(GL_FRONT_AND_BACK, GL_LINE);

    glEnable(GL_LINE_STIPPLE);
    glLineStipple(1, $1111);
    glLineWidth(0.3);

    q := gluNewQuadric;
    glTranslatef(PointofRotation[0], PointofRotation[1], PointofRotation[2]);
    glColor3f(0.5, 0.5, 0.5);
    gluSphere(q, RadiusArcBall, 12, 12);
    gluDeleteQuadric(q);
  glPopMatrix;
  glPopAttrib;

//  XYArc(RadiusArcBall, -20, 20, 10, PointofRotation, ArcBallEnd);
//  glPushMatrix;
//    glColor3f(0, 1, 0);
//    glBegin(GL_LINE_STRIP);
//      for i := 0 to 10 - 1 do
//      begin
//        glVertex3f(V[i][0], V[i][1], V[i][2]);
//      end;
//    glEnd;
//  glPopMatrix;
end;

end.

