unit CamerAHUtils;

interface

uses
  Types, Classes, Controls;

type

  TglMouse = class(TObject)
  public
    Anker, ziehen: TPoint;
    IsLupeZiehen: Boolean;
    Height, Width: Integer;

    constructor Create;
    procedure An(X, Y: Integer; Shift: TShiftState; W, H: integer);
    procedure Bewegen(X, Y: Integer);
    function  Aus(X, Y: Integer; Button: TMouseButton): Boolean;
    function  FAktivFlche: TRect;
    property  AktivFlche: TRect read FAktivFlche;
  end;

  TStopWatchMethode = (su_DateTime, su_TickCount, su_Performance);
  TStopWatch = class
  //http://www.delphipraxis.net/134663-tstopuhr-kleine-klasse-zur-zeitmessung.html
    private
      FStoppedTime : Int64;
      FStartValue : Int64;
      FStopValue : Int64;
      FStartTime : TDateTime;
      FStopTime : TDateTime;
      FMethode : TStopWatchMethode;
      FMethodeInUse : TStopWatchMethode;
    protected
      function GetStoppedTimeStr: String;
    public
      Constructor Create;
      Destructor Destroy; Override;
      procedure Start;
      procedure Stop;
      function  RunningTime: Int64;
      property StartTime : TDateTime read FStartTime;
      property StopTime : TDateTime read FStopTime;
      property StoppedTime : Int64 read FStoppedTime;
      property StoppedTimeStr : String read GetStoppedTimeStr;
      property MeasureMethode : TStopWatchMethode Read FMethode Write FMethode;
    end;

implementation

uses
  SysUtils, Windows;

// ******************************** TglMouse *******************************
{$REGION 'TglMouse'}

constructor TglMouse.Create;
begin
  inherited;
  IsLupeZiehen := false;
end;

function TglMouse.FAktivFlche: TRect;
var
  xx, yy: Integer;
begin
  if Ziehen.X < Anker.X then
  begin
    xx       := Anker.X;
    Anker.X  := Ziehen.X;
    Ziehen.X := xx;
  end;

  if Ziehen.Y < Anker.Y then
  begin
    yy       := Anker.Y;
    Anker.Y  := Ziehen.Y;
    Ziehen.Y := yy;
  end;

  Result := Rect(Anker.X, Anker.Y, Ziehen.X, Ziehen.Y);
end;

procedure TglMouse.An(X, Y: Integer; Shift: TShiftState; W, H: integer);
begin
  if (Shift = [ssLeft]) then
  begin
    Height := H;
    Width := W;
    IsLupeZiehen := true;
    Anker        := Point(X, Y);
    ziehen       := Point(X, Y);
  end;
end;

procedure TglMouse.Bewegen(X, Y: Integer);
begin
  if IsLupeZiehen then
  begin
      ziehen.X := X;
      ziehen.Y := Y;
  end;
end;

function TglMouse.Aus(X, Y: Integer; Button: TMouseButton): Boolean;
begin
  Result := IsLupeZiehen;
  if (Button = mbLeft) and IsLupeZiehen then
    IsLupeZiehen := false;
end;
{$ENDREGION}
// ******************************** TglMouse *******************************

// ******************************* TStopWatch ******************************
{$REGION 'TStopWatch'}

var
  g_QPCFrequenz : Int64 = 0; // Vorschlag shmia, siehe Post

constructor TStopWatch.Create;
begin
  FStartTime := Now;
  FStopTime := FStartTime;
  FStoppedTime := 0;
  FStartValue := 0;
  FStopValue := 0;
  FMethode := su_DateTime;
  FMethodeInUse := su_DateTime;
end;

destructor TStopWatch.Destroy;
begin
  inherited;
end;

function TStopWatch.GetStoppedTimeStr: String;
begin
  Result := Format('%.0n ms',[FStoppedTime / 1]);
end;

function TStopWatch.RunningTime: Int64;
begin
  Result := GetTickCount - FStartValue;
end;

procedure TStopWatch.Start;
begin
  FMethodeInUse := FMethode;
  case FMethodeInUse of
    su_DateTime : FStartTime := Now;
    su_TickCount : FStartValue := GetTickCount;
    su_Performance : QueryPerformanceCounter(FStartValue);
  end;
end;

procedure TStopWatch.Stop;
const
  msPerDay = 86400000;
var
  Days, Hour, Min, Sec, MSec: Word;
begin
  case FMethodeInUse of
    su_DateTime :
    begin
      FStopTime := Now;
      DecodeTime(FStopTime - FStartTime, Hour, Min, Sec, MSec);
      Days := Trunc(FStopTime) - Trunc(FStartTime);
      FStoppedTime := MSec + (Sec * 1000) + (Min * 60 * 1000) + (Hour * 60 * 60 * 1000) +
                      (Days * msPerDay);
    end;
    su_TickCount :
    begin
      FStopValue := GetTickCount;
      FStoppedTime := FStopValue - FStartValue;
    end;
    su_Performance :
    begin
      QueryPerformanceCounter(FStopValue);
      FStoppedTime := (FStopValue - FStartValue) * 1000 div g_QPCFrequenz;
    end;
  end;
end;
{$ENDREGION}
// ******************************* TStopWatch ******************************

initialization
   QueryPerformanceFrequency(g_QPCFrequenz);
end.
