unit figures;

interface

uses
  CamerAHTypes, CamerAHGeometry;

Const voffset : TVector3f = (0.0, 0.0, 0.0);
      toffset : TVector2f = (0.0, 0.0);
      tscale  : TVector2f = (1.0, 1.0);

procedure Sphere(c: TVector; r: Single; n: Integer = 40);

procedure Cube(dx, dy, dz: Single; inverted: boolean; voffset: TVector3f; toffset, tscale: TVector2f);
// Cube(1.0, 1.0, 1.0, false, voffset, toffset, tscale);

procedure CoordPlane;

implementation

uses
  dglOpenGL;

procedure Sphere(c: TVector; r: Single; n: Integer = 40);
const
  TWOPI = PI*2;
  PID2 = PI/2;
var
  i,j: Integer;
  theta1,theta2,theta3: Single;
  e,p:TVector3f;
begin
  if (r < 0) then r := -r;
  if (n < 0) then n := -n;
  if (n < 4) or (r <= 0) then
  begin
    glBegin(GL_POINTS);
      glVertex3f(c[0],c[1],c[2]);
    glEnd;
    exit;
  end;

  for j:=0 to n div 2 do
  begin
    theta1 := j * TWOPI / n - PID2;
    theta2 := (j + 1) * TWOPI / n - PID2;

    glBegin(GL_QUAD_STRIP);
    for i:=0 to n do
    begin
      theta3 := i * TWOPI / n;

      e[0] := cos(theta2) * cos(theta3);
      e[1] := sin(theta2);
      e[2] := cos(theta2) * sin(theta3);
      p[0] := c[0] + r * e[0];
      p[1] := c[1] + r * e[1];
      p[2] := c[2] + r * e[2];

      glNormal3f(e[0],e[1],e[2]);
      glTexCoord2f(i/n,2*(j+1)/n);
      glVertex3f(p[0],p[1],p[2]);

      e[0] := cos(theta1) * cos(theta3);
      e[1] := sin(theta1);
      e[2] := cos(theta1) * sin(theta3);
      p[0] := c[0] + r * e[0];
      p[1] := c[1] + r * e[1];
      p[2] := c[2] + r * e[2];

      glNormal3f(e[0],e[1],e[2]);
      glTexCoord2f(i/n,2*j/n);
      glVertex3f(p[0],p[1],p[2]);
    end;
   glEnd;
  end;
end;

procedure Cube(dx, dy, dz: Single; inverted: boolean;
                        voffset: TVector3f; toffset, tscale: TVector2f);
Var i : shortint;
Begin
  If (inverted)
    Then i := -1
    Else i := +1;

  dx := 0.5 * dx;
  dy := 0.5 * dy;
  dz := 0.5 * dz;

  glBegin(GL_QUADS);
    // left (inverted: right)
    glNormal3f(-1, 0, 0);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(-dx*i + voffset[0], -dy + voffset[1], -dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(-dx*i + voffset[0], -dy + voffset[1], +dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx*i + voffset[0], +dy + voffset[1], +dz + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx*i + voffset[0], +dy + voffset[1], -dz + voffset[2]);

    // right (inverted: left)
    glNormal3f(1, 0, 0);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(+dx*i + voffset[0], -dy + voffset[1], +dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(+dx*i + voffset[0], -dy + voffset[1], -dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx*i + voffset[0], +dy + voffset[1], -dz + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx*i + voffset[0], +dy + voffset[1], +dz + voffset[2]);

    // top (inverted: bottom)
    glNormal3f(0, 1, 0);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(-dx + voffset[0], +dy*i + voffset[1], +dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(+dx + voffset[0], +dy*i + voffset[1], +dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx + voffset[0], +dy*i + voffset[1], -dz + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx + voffset[0], +dy*i + voffset[1], -dz + voffset[2]);

    // bottom (inverted: top)
    glNormal3f(0, -1, 0);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(-dx + voffset[0], -dy*i + voffset[1], -dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(+dx + voffset[0], -dy*i + voffset[1], -dz + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx + voffset[0], -dy*i + voffset[1], +dz + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx + voffset[0], -dy*i + voffset[1], +dz + voffset[2]);

    // front (inverted: back)
    glNormal3f(0, 0, 1);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(-dx + voffset[0], -dy + voffset[1], +dz*i + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(+dx + voffset[0], -dy + voffset[1], +dz*i + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx + voffset[0], +dy + voffset[1], +dz*i + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx + voffset[0], +dy + voffset[1], +dz*i + voffset[2]);

    // back (inverted: front)
    glNormal3f(0, 0, -1);
    glTexCoord2f(            toffset[0],             toffset[1]);
    glVertex3f(+dx + voffset[0], -dy + voffset[1], -dz*i + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0],             toffset[1]);
    glVertex3f(-dx + voffset[0], -dy + voffset[1], -dz*i + voffset[2]);
    glTexCoord2f(tscale[0] + toffset[0], tscale[1] + toffset[1]);
    glVertex3f(-dx + voffset[0], +dy + voffset[1], -dz*i + voffset[2]);
    glTexCoord2f(            toffset[0], tscale[1] + toffset[1]);
    glVertex3f(+dx + voffset[0], +dy + voffset[1], -dz*i + voffset[2]);
  glEnd;
End;

procedure CoordPlane;
begin
  glBegin(GL_LINES);
    glColor3f(0, 1, 0);
    glVertex3f(0,0,0);
    glVertex3f(0,1,0);

    glColor3f(0, 0, 1);
    glVertex3f(0,0,0);
    glVertex3f(0,0,1);

    glColor3f(1, 0, 0);
    glVertex3f(0,0,0);
    glVertex3f(1,0,0);
  glEnd;
end;

end.
